import hashlib
import hmac


def validate_webhook_signature(
    signature_header: str, raw_body: bytes, secret_key: str
) -> bool:
    """
    Validate a Paddle webhook signature.
    More info: https://developer.paddle.com/webhooks/signature-verification

    :param signature_header: The Paddle-Signature header value.
    In Django for example, this is request.META['HTTP_PADDLE_SIGNATURE'].
    :param raw_body: The raw body of the request.
    :param secret_key: The secret key used to generate the signature.
    """
    ts_part, h1_part = signature_header.split(";")
    var, timestamp = ts_part.split("=")
    var, signature = h1_part.split("=")

    signed_payload = ":".join([timestamp, raw_body.decode("utf-8")])

    # Paddle generates signatures using a keyed-hash message authentication code (HMAC) with SHA256 and a secret key.
    computed_signature = hmac.new(
        key=secret_key.encode("utf-8"),
        msg=signed_payload.encode("utf-8"),
        digestmod=hashlib.sha256,
    ).hexdigest()

    # Compare the computed signature with the signature extracted from the Paddle-Signature header.
    return hmac.compare_digest(computed_signature, signature)


def paddle_create_signature_header(
    raw_body: bytes, secret_key: str, timestamp: str = "12345"
) -> str:
    """Create a paddle signature header to help test your webhook implementation.

    Example usage:
        data:str = "{json_data_string}"
        header = paddle_create_signature_header(data.encode())
        # client is your test client posting in your webhook route
        r0 = client.post("/paddle_webhook/", data=data, headers={"Paddle-Signature": header, "Content-Type": "application/json"}
    """
    # ts=1671552777;h1=eb4d0dc8853be92b7f063b9f3ba5233eb920a09459b6e6b2c26705b4364db151
    signed_payload = ":".join([timestamp, raw_body.decode("utf-8")])
    computed_signature = hmac.new(
        key=secret_key.encode("utf-8"),
        msg=signed_payload.encode("utf-8"),
        digestmod=hashlib.sha256,
    ).hexdigest()

    header = f"ts={timestamp};h1={computed_signature}"
    return header



from typing import Callable

from paddle_billing_client.models.common import Paginate


def paginate(get: Callable, **kwargs):
    response = get(**kwargs)
    yield response

    if response.meta.pagination is None:
        raise Exception("Pagination is not supported for this endpoint")

    while response.meta.pagination.has_more and response.meta.pagination.next:
        response = get(paginate=Paginate(next=response.meta.pagination.next))
        yield response



from __future__ import annotations

from typing import Dict, List, Literal, Optional

from datetime import datetime

from pydantic import ConfigDict, model_validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.models.common import ImportMeta
from paddle_billing_client.models.price import Price


class ProductBase(BaseModel):
    name: str
    tax_category: Literal[
        "digital-goods",
        "ebooks",
        "implementation-services",
        "professional-services",
        "saas",
        "software-programming-services",
        "standard",
        "training-services",
        "website-hosting",
    ]
    description: str | None = None
    image_url: str | None = None
    custom_data: dict[str, str] | None = None
    status: Literal["active", "archived"] | None = None
    type: Literal["custom", "standard"] | None = None


class Product(ProductBase):
    id: str
    created_at: datetime | None = None
    prices: list[Price] | None = None
    import_meta: ImportMeta | None = None


class ProductQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: str | None = None
    # Include related entities in the response.
    include: Literal["prices"] | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: str | None = None
    # Return entities that match the specified tax category.
    tax_category: None | (
        Literal[
            "digital-goods",
            "ebooks",
            "implementation-services",
            "professional-services",
            "saas",
            "software-programming-services",
            "standard",
            "training-services",
            "website-hosting",
        ]
    ) = None

    model_config = ConfigDict(extra="forbid")

    @model_validator(mode="after")
    def check_status(self):
        valid_statuses = ["active", "archived"]
        if self.status and not all(
            [s in valid_statuses for s in self.status.split(",")]
        ):
            raise ValueError(
                f"Query param invalid status: {self.status}, allowed values: {valid_statuses}"
            )
        return self


class ProductResponse(PaddleResponse):
    data: Product


class ProductsResponse(PaddleResponse):
    data: list[Product]


class ProductRequest(ProductBase):
    pass



from apiclient import exceptions
from apiclient.error_handlers import BaseErrorHandler
from apiclient.response import Response


class VerboseErrorHandler(BaseErrorHandler):
    @staticmethod
    def get_exception(response: Response) -> exceptions.APIRequestError:
        status_code = response.get_status_code()
        exception_class = exceptions.UnexpectedError

        if 300 <= status_code < 400:
            exception_class = exceptions.RedirectionError
        elif 400 <= status_code < 500:
            exception_class = exceptions.ClientError
        elif 500 <= status_code < 600:
            exception_class = exceptions.ServerError

        return exception_class(
            message=(
                f"{status_code} Error: {response.get_status_reason()} "
                f"for url: {response.get_requested_url()}"
                f"with raw data: {response.get_raw_data()}"
            ),
            status_code=status_code,
            info=response.get_raw_data(),
        )



from __future__ import annotations

from typing import Literal

from datetime import datetime

from pydantic import ConfigDict, Field, model_validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.utils import parse_event_to_model


class NotificationIdPathParams(BaseModel):
    notificationId: str


class NotificationPayload(BaseModel):
    notification_id: str
    event_id: str
    event_type: str
    data: dict | BaseModel
    occurred_at: datetime

    @model_validator(mode="after")
    def check_status(self):
        self.data = parse_event_to_model(self.event_type, self.data)
        return self


class Notification(BaseModel):
    id: str
    type: str
    status: str
    payload: NotificationPayload
    occurred_at: datetime
    delivered_at: datetime | None = None
    replayed_at: str | None = None
    origin: str
    last_attempt_at: datetime | None = None
    retry_at: str | None = None
    times_attempted: int
    notification_setting_id: str


class NotificationQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Return entities related to the specified notification destination.
    # Use a comma separated list to specify multiple notification destination IDs.
    notification_setting_id: str | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None
    # Return entities that match a search query. Searches id and type fields.
    search: str | None = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: str | None = None
    # Return entities that contain the Paddle ID specified. Pass a transaction, customer, or subscription ID.
    filter: str | None = None
    # Return entities up to a specific time.
    to: str | None = None
    # Return entities from a specific time.
    from_field: str | None = Field(None, alias="from")

    model_config = ConfigDict(extra="forbid")

    @model_validator(mode="after")
    def check_status(self):
        valid_statuses = ["delivered", "failed", "needs_retry", "not_attempted"]
        if self.status and not all(
            [s in valid_statuses for s in self.status.split(",")]
        ):
            raise ValueError(
                f"Query param invalid status: {self.status}, allowed values: {valid_statuses}"
            )
        return self


class NotificationResponse(PaddleResponse):
    data: Notification


class NotificationsResponse(PaddleResponse):
    data: list[Notification]


class NotificationReplay(BaseModel):
    event_id: str | None = None


class NotificationReplayResponse(PaddleResponse):
    data: NotificationReplay



from __future__ import annotations

from typing import Dict, List, Literal, Optional

from pydantic import ConfigDict, model_validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.models.common import ImportMeta


class UnitPrice(BaseModel):
    amount: str
    currency_code: str


class Quantity(BaseModel):
    minimum: int
    maximum: int


class BillingCycle(BaseModel):
    interval: str
    frequency: int


class UnitPriceOverride(BaseModel):
    country_codes: list[str]
    unit_price: UnitPrice


class TrialPeriod(BaseModel):
    interval: str
    frequency: int


class PriceBase(BaseModel):
    description: str
    product_id: str | None = None
    billing_cycle: BillingCycle | None = None
    trial_period: TrialPeriod | None = None
    tax_mode: str
    unit_price_overrides: list[UnitPriceOverride] | None = None
    quantity: Quantity | None = None
    custom_data: dict[str, str] | None = None
    type: Literal["custom", "standard"] | None = None
    name: str | None = None


class Price(PriceBase):
    id: str
    unit_price: UnitPrice
    status: str | None = None
    import_meta: ImportMeta | None = None


class PriceQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: str | None = None
    # Include related entities in the response.
    include: Literal["product"] | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: str | None = None
    # Determine whether returned entities are for recurring prices (true) or one-time prices (false).
    recurring: bool | None = None

    model_config = ConfigDict(extra="forbid")

    @model_validator(mode="after")
    def check_status(self):
        valid_statuses = ["active", "archived"]
        if self.status and not all(
            [s in valid_statuses for s in self.status.split(",")]
        ):
            raise ValueError(
                f"Query param invalid status: {self.status}, allowed values: {valid_statuses}"
            )
        return self


class PriceResponse(PaddleResponse):
    data: Price


class PricesResponse(PaddleResponse):
    data: list[Price]


class PriceRequest(PriceBase):
    pass



from __future__ import annotations

from typing import Dict, List, Literal, Optional

from datetime import datetime

from pydantic import ConfigDict, model_validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.models.common import ImportMeta


class DiscountBase(BaseModel):
    amount: str
    description: str
    type: Literal["flat", "flat_per_seat", "percentage"]
    enabled_for_checkout: bool
    code: str
    currency_code: str | None = None
    recur: bool
    maximum_recurring_intervals: int | None = None
    usage_limit: int | None = None
    restrict_to: list[str] | None = None
    expires_at: datetime | None = None
    status: Literal["active", "archived", "expired", "used"] | None = None
    custom_data: dict[str, str] | None = None


class Discount(DiscountBase):
    id: str
    times_used: int
    created_at: datetime | None = None
    updated_at: datetime | None = None
    external_id: str | None = None
    import_meta: ImportMeta | None = None


class DiscountQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Return entities that match the discount code. Use a comma separated list to specify multiple discount codes.
    code: str | None = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: str | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: str | None = None

    model_config = ConfigDict(extra="forbid")

    @model_validator(mode="after")
    def check_status(self):
        valid_statuses = ["active", "archived", "expired", "used"]
        if self.status and not all(
            [s in valid_statuses for s in self.status.split(",")]
        ):
            raise ValueError(
                f"Query param invalid status: {self.status}, allowed values: {valid_statuses}"
            )
        return self


class DiscountResponse(PaddleResponse):
    data: Discount


class DiscountsResponse(PaddleResponse):
    data: list[Discount]


class DiscountRequest(DiscountBase):
    pass



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, ConfigDict


class PaymentTerms(BaseModel):
    interval: Literal["day", "week", "month", "year"]
    frequency: int


class BillingPeriod(BaseModel):
    ends_at: str
    starts_at: str


class BillingDetails(BaseModel):
    enable_checkout: bool
    purchase_order_number: str | None = None
    additional_information: str | None = None
    payment_terms: PaymentTerms


class Paginate(BaseModel):
    next: str | None = None

    model_config = ConfigDict(extra="forbid")


class ImportMeta(BaseModel):
    external_id: str | None = None
    imported_from: str



from typing import Optional

from pydantic import BaseModel, ConfigDict


class LazyBaseModel(BaseModel):
    model_config = ConfigDict(extra="allow")


class Pagination(LazyBaseModel):
    per_page: int
    estimated_total: int
    next: Optional[str] = None
    has_more: bool


class Meta(LazyBaseModel):
    pagination: Optional[Pagination] = None
    request_id: str


class PaddleResponse(LazyBaseModel):
    meta: Meta



# type: ignore[attr-defined]
"""Python wrapper around the new Paddle Billing API"""

import sys

if sys.version_info >= (3, 8):
    from importlib import metadata as importlib_metadata
else:
    import importlib_metadata  # pragma: no cover


def get_version() -> str:
    try:
        return importlib_metadata.version(__name__)
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


version: str = get_version()



from __future__ import annotations

from typing import List, Literal, Optional

from datetime import datetime

from pydantic import model_validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.utils import parse_event_to_model


class EventType(BaseModel):
    name: str
    description: str
    group: str
    available_versions: list[int]


class EventTypesResponse(PaddleResponse):
    data: list[EventType]


class EventQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None


class Event(BaseModel):
    notification_id: str | None = None
    event_id: str
    event_type: str
    data: dict | BaseModel  # | Subscription | Transaction | Customer | Product | Price | Address | Business | Adjustment
    occurred_at: datetime

    @model_validator(mode="after")
    def check_status(self):
        self.data = parse_event_to_model(self.event_type, self.data)
        return self


class EventsResponse(PaddleResponse):
    data: list[Event]



from __future__ import annotations

from typing import List, Literal, Optional

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse


class SubscribedEvent(BaseModel):
    name: str
    description: str
    group: str
    available_versions: list[int]


class NotificationSettingBase(BaseModel):
    description: str
    destination: str
    subscribed_events: list[SubscribedEvent]
    type: Literal["email", "url"] | None = None
    active: bool
    api_version: int
    include_sensitive_fields: bool


class NotificationSetting(NotificationSettingBase):
    id: str
    endpoint_secret_key: str


class NotificationSettingResponse(PaddleResponse):
    data: NotificationSetting


class NotificationSettingsResponse(PaddleResponse):
    data: list[NotificationSetting]


class NotificationSettingRequest(NotificationSettingBase):
    subscribed_events: list[str]



import functools

from paddle_billing_client.models.address import Address
from paddle_billing_client.models.adjustment import Adjustment
from paddle_billing_client.models.business import Business
from paddle_billing_client.models.customer import Customer
from paddle_billing_client.models.price import Price
from paddle_billing_client.models.product import Product
from paddle_billing_client.models.subscription import Subscription
from paddle_billing_client.models.transaction import Transaction


def parse_event_to_model(event_type: str, data):
    if not isinstance(data, dict):
        return data
    if event_type.startswith("subscription"):
        return Subscription(**data)
    elif event_type.startswith("transaction"):
        return Transaction(**data)
    elif event_type.startswith("customer"):
        return Customer(**data)
    elif event_type.startswith("product"):
        return Product(**data)
    elif event_type.startswith("price"):
        return Price(**data)
    elif event_type.startswith("address"):
        return Address(**data)
    elif event_type.startswith("business"):
        return Business(**data)
    elif event_type.startswith("adjustment"):
        return Adjustment(**data)
    return data
